using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace HHHHControls
{
	/// <summary>
	/// The DBAnimUnsetStyle enumeration specifies how the control should draw itself when no bitmap
	/// has been assigned to the _Frame property.  UnsetStyle.Empty specifies that the control should 
	/// fill itself with its BackColor, while UnsetStyle.Cross specifies that the control should draw
	/// a large "X" across its face.
	/// </summary>
	public enum DBAnimUnsetStyle
	{
		Empty,
		Cross
	}

	/// <summary>
	/// DBAnim is a double-buffered animation control.  The basic usage pattern is:
	/// 1. Create a bitmap in which to draw your frames.  DBAnim will automatically stretch your bitmap
	///	   to fit its own size, so create one that is the same size as the control in order to get a 1:1
	///	   display.
	///	2. Assign your bitmap to DBAnim.Frame:
	///	   DBAnim1.Frame = mybitmap;
	///	3. Put whatever you want into the bitmap.
	///	4. Call DBAnim.ShowFrame() to cause your bitmap to be displayed on the screen.  Note that if the control
	///	   processes a resize event while you are updating the bitmap, intermediate results may be displayed.
	///	5. Repeat steps 3-4 as necessary.  You can re-assign to the Frame member whenever you need to, in
	///	   order to handle resize events, etc.
	/// </summary>
	[
		ToolboxBitmapAttribute(typeof(HHHHControls.DBAnim), "DBAnim.bmp")
	]
	public class DBAnim : System.Windows.Forms.Control
	{
		private Bitmap _Frame;
		/// <summary>
		/// Set Frame to the bitmap you're using as your animation cell.  After setting Frame
		/// or changing the contents of the bitmap that Frame refers to, use the Update() method
		/// to cause the DBAnim control to re-draw itself using the new or modified bitmap.
		/// </summary>
		[
			Browsable(false)
		]
		public Bitmap Frame 
		{
			get { return _Frame; }
			set { _Frame = value; }
		}

		private DBAnimUnsetStyle _Style;
		[
			Browsable(true),
			CategoryAttribute("DBAnim"),
			Description("The control's behavior when no bitmap is assigned; See DBAnim.UnsetStyle.")
		]
		public DBAnimUnsetStyle Style 
		{
			get { return _Style; }
			set { _Style = value; }
		}
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public DBAnim()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.Name = "DBAnim";
			this.Size = new Size(128,128);
			this.SetStyle(ControlStyles.DoubleBuffer, true);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			this.SetStyle(ControlStyles.UserPaint, true);

			this._Style = DBAnimUnsetStyle.Cross;
			this._Frame = (Bitmap)null;
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			if(_Frame != (Bitmap)null) 
			{
				e.Graphics.InterpolationMode = InterpolationMode.Bicubic;
				e.Graphics.DrawImage(_Frame, new Rectangle(0, 0, this.Width, this.Height));
			} 
			else
			{
				using (SolidBrush B = new SolidBrush(this.BackColor))
				using (SolidBrush F = new SolidBrush(this.ForeColor))
				{
					// Empty out the control
					e.Graphics.FillRectangle(B, 0,0,this.Width, this.Height);
					// Draw a big "X" if necessary.
					if(_Style == DBAnimUnsetStyle.Cross) 
					{
						int x0 =  0 * this.Width / 16,
							x1 =  1 * this.Width / 16, 
							x2 =  7 * this.Width / 16, 
							x3 =  8 * this.Width / 16,
							x4 =  9 * this.Width / 16,
							x5 = 15 * this.Width / 16,
							xW = 16 * this.Width / 16,
							y0 =  0 * this.Height / 16,
							y1 =  1 * this.Height / 16,
							y2 =  7 * this.Height / 16,
							y3 =  8 * this.Height / 16,
							y4 =  9 * this.Height / 16,
							y5 = 15 * this.Height / 16,
							yW = 16 * this.Height / 16;
						GraphicsPath CrossPath = new GraphicsPath(
							new Point[] {
								new Point(x0, y0),
								new Point(x1, y0),
								new Point(x3, y2),
								new Point(x5, y0),
								new Point(xW, y0),
								new Point(xW, y1),
								new Point(x4, y3),
								new Point(xW, y5),
								new Point(xW, yW),
								new Point(x5, yW),
								new Point(x3, y4),
								new Point(x1, yW),
								new Point(x0, yW),
								new Point(x0, y5),
								new Point(x2, y3),
								new Point(x0, y1),
								new Point(x0, y0)
							},
							new byte[] {
								(byte)PathPointType.Start,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line,
								(byte)PathPointType.Line
							},
							FillMode.Alternate);
						e.Graphics.FillPath(F, CrossPath);
					}
				}
			}
		}

		private void DBAnim_Resize(object sender, System.EventArgs e)
		{
			this.ShowFrame();
		}

		/// <summary>
		/// Re-draw the control on the screen.  Invoke this method after assigning to the Frame property,
		/// or after making changes to the bitmap referenced by the Frame property.
		/// </summary>
		public void ShowFrame()
		{
			this.Invalidate();	// send WM_PAINT
			this.Update();		// push WM_PAINT message to the head of the queue
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// DBAnim
			// 
			this.Resize += new System.EventHandler(this.DBAnim_Resize);

		}
		#endregion
	}
}
