using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace HHHHControls
{
	/// <summary>
	/// Summary description for ImagePreviewPanel.
	/// </summary>
	[
	ToolboxBitmapAttribute(typeof(HHHHControls.ImagePreviewPanel),
		"ImagePreviewPanel.bmp")
	]
	public class ImagePreviewPanel : System.Windows.Forms.Control
	{
		/// <summary>
		/// Here are members corresponding to externally-usable properties
		/// </summary>
		private Color _HighlightColor;	// Color of the highlight gradient around the selected image.
		private int _Count;				// how many images are being shown in the control
		private int _Selected;			// which of the Items is the "selected" one.
		private int _Margin;			// amount of padding between thumbnails and boundries of their cells
		private bool _Highlight;		// whether we're actually drawing the selection highlight
		private bool _Border;			// whether we're drawing a border.
		private Bitmap[] _Items;		// managed through a this[int] accessor.

		/// <summary>
		/// Here are mambers used for internally managing the control and optimizing drawing.
		/// </summary>
		private Bitmap BackgroundTile, HighlightTile;

		/// <summary>
		/// Color of the highlight gradient drawn around the selected image.
		/// </summary>
		[
		Browsable(true),
		CategoryAttribute("ImagePreviewControl"),
		Description("Color of the highlight gradient drawn around the selected image.")
		]
		public Color HighlightColor 
		{
			get { return _HighlightColor; }
			set { _HighlightColor = value; CreateTiles(); }
		}
		/// <summary>
		/// Get and set Images into the control. 
		/// </summary>
		[
		Browsable(true),
		CategoryAttribute("ImagePreviewControl"),
		Description("Get and set Images into the control")
		]
		public Image this[int i] 
		{
			get { return (Image)(_Items[i]); }
			set {
				if(i < 0 || i >= _Count)
					throw new System.ArgumentOutOfRangeException("value",value,String.Format("Index {0} is out of range",value.ToString()));
				if(value != (Image)null) 
				{
					_Items[i] = GetThumbnailFromImage(value);
					return; 
				}
			}
		}
		/// <summary>
		/// Number of images to display.  When this value changes, you must re-set all
		/// the items in the control.
		/// </summary>
		[
		Browsable(true),
		CategoryAttribute("ImagePreviewControl"),
		Description("Number of images to display.  If this value changes, you must re-set all the items in the control.")
		]
		public int Count 
		{
			get { return _Count; }
			set 
			{
				if(value == _Count)	// no actual work to be done
					  return;
				// but if the count is actually changing, then:
					// dispose of any thumbnails in _Items,
					// reset _Count,
					// re-allocate _Items to the new size,
					// set SelectedIndex to -1
					// re-create background and highlight tiles
				for(int i = 0; i < _Count; i++)
					if(_Items[i] != (Bitmap)null)
						_Items[i].Dispose();
				_Count = value; 
				_Items = new Bitmap[_Count];
				for(int i = 0; i < _Count; i++)
					_Items[i] = (Bitmap)null;
				_Selected = -1;
				CreateTiles();
			}
		}
		/// <summary>
		/// Which item, if any, is currently selected.
		/// </summary>
		[
		Browsable(true),
		CategoryAttribute("ImagePreviewControl"),
		Description("Which item, if any, is currently selected.")
		]
		public int SelectedIndex 
		{
			get { return _Selected; }
			set 
			{
				if(_Selected == value)	// no work to be done.
					  return;
				if(value < 0 || value >= _Items.Length)
					throw new ArgumentOutOfRangeException("value",value,String.Format("Index {0} is out of range",value.ToString()));
				_Selected = value;		// reset the selection.
				this.Invalidate();		// redraw.
				this.Update();
			}
		}
		/// <summary>
		/// Number of pixels of blank space between images and the edges of their
		/// cells within the control
		/// </summary>
		[
		Browsable(true),
		CategoryAttribute("ImagePreviewControl"),
		Description("Number of pixels of blank space between images and the edges of their cells within the control")
		]
		public int Margin 
		{
			get { return _Margin; }
			set { _Margin = value; }
		}
		/// <summary>
		/// Whether to highlight the selected item.
		/// </summary>
		[
		Browsable(true),
		CategoryAttribute("ImagePreviewControl"),
		Description("Whether to highlight the selected item")
		]
		public bool HighlightSelected 
		{
			get { return _Highlight; }
			set { _Highlight = value; }
		}
		
		/// <summary>
		/// Whether to draw a border around the control.
		/// </summary>
		[
		Browsable(true),
		CategoryAttribute("ImagePreviewControl"),
		Description("Whether to draw border around the control")
		]
		public bool Border 
		{
			get { return _Border; }
			set { _Border = value; }
		}
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public ImagePreviewPanel()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// set default values
			this._Count = 3;
			this._Items = new Bitmap[3]{(Bitmap)null,(Bitmap)null,(Bitmap)null};
			this._Highlight = true;
			this._Border = false;
			this._HighlightColor = this.ForeColor;
			this._Margin = 10;
			this._Selected = -1; // hack.  Really want C# 2.0 nullable int for this.

			// Required stuff.
			this.Name = "ImagePreviewPanel";
			this.Size = new Size(288,96);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			this.SetStyle(ControlStyles.UserPaint, true);

			// create background tile and highlight tile.
			CreateTiles();

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();

				// Dispose of our tiles and thumbnails:
				if(_Count > 0) 
				{
					for(int i = 0; i < _Count; i++)
						if(_Items[i] != (Bitmap)null)
							_Items[i].Dispose();
				}
				if(BackgroundTile != (Bitmap)null)
					BackgroundTile.Dispose();
				if(HighlightTile != (Bitmap)null)
					HighlightTile.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// ImagePreviewPanel
			// 
			this.Resize += new System.EventHandler(this.ImagePreviewPanel_Resize);
			this.BackColorChanged += new System.EventHandler(this.ImagePreviewPanel_BackColorChanged);

		}
		#endregion

		private void CreateTiles() 
		{
			if(this.Width == 0 || this.Height == 0)	// safety check.  Can't create the tiles yet if the control has no size.
				return;

			if(this.BackgroundTile != (Bitmap)null)
				this.BackgroundTile.Dispose();
			if(this.HighlightTile != (Bitmap)null)
				this.HighlightTile.Dispose();

			int TileWidth  = this.Width  / _Count,
				TileHeight = this.Height;

			this.BackgroundTile = new Bitmap(TileWidth, TileHeight);
			this.HighlightTile  = new Bitmap(TileWidth, TileHeight);
			using(Graphics G = Graphics.FromImage(BackgroundTile))
			using(SolidBrush B = new SolidBrush(this.BackColor)) 
			{
				G.FillRectangle(B, 0,0, TileWidth-1, TileHeight-1);
			}
			using(Graphics G = Graphics.FromImage(HighlightTile)) 
			{
				Point[] path = new Point[4]{new Point(0,0),
											new Point(TileWidth-1, 0),
											new Point(TileWidth-1, TileHeight-1),
											new Point(0, TileHeight-1) };
				using(PathGradientBrush B = new PathGradientBrush(path))
				{
					ColorBlend CB = new ColorBlend(4);
					CB.Colors = new Color[4] { this.BackColor, this.HighlightColor, this.HighlightColor, this.HighlightColor};
					CB.Positions = new float[4] { 0.0f, 0.33f, 0.66f, 1.0f };
					B.InterpolationColors = CB;
					G.FillPath(B, new GraphicsPath(path, new byte[4]{(byte)PathPointType.Start, (byte)PathPointType.Line, (byte)PathPointType.Line, (byte)PathPointType.Line}));
				}
			}
		}

		/// <summary>
		/// Add an image to the control by specifying its index and a filename to load the image from.
		/// </summary>
		/// <param name="index"></param>
		/// <param name="filename"></param>
		public void AddFileAt(int index, string filename) 
		{
			if(index < 0 || index >= _Count)
				throw new System.ArgumentOutOfRangeException("index",index,String.Format("Index {0} is out of range",index.ToString()));
			if(filename != (string)null) 
			{
				_Items[index] = GetThumbnailFromFilename(filename);
				return; 
			}
		}
		/// <summary>
		/// Add an image to the left side of the control by specifying a filename.  Existing images are shifted to the right.
		/// </summary>
		/// <param name="filename"></param>
		public void AddFileLeft(string filename) 
		{
			for(int i = _Items.Length-1; i > 0; i--)	// shift everything to the right.
				_Items[i] = _Items[i-1];	
			AddFileAt(0, filename);						// put a new thumbnail on the left.
		}
		/// <summary>
		/// Add an image to the right side of the control by specifying a filename.  Existing images are shifted to the left.
		/// </summary>
		/// <param name="filename"></param>
		public void AddFileRight(string filename)
		{
			for(int i = 0; i < _Items.Length-1; i++)	// shift everything to the left.
				_Items[i] = _Items[i+1];
			AddFileAt(_Items.Length-1, filename);		// put a new thumbnail on the right.
		}

		/// <summary>
		/// Generate a thumbnail image by loading an image file from a filename and scaling it to the proper size for the control.
		/// </summary>
		/// <param name="filename"></param>
		/// <returns></returns>
		private Bitmap GetThumbnailFromFilename(string filename) 
		{
			Bitmap image = new Bitmap(filename);	// load the image file.
			if(image != (Bitmap)null) 
			{
				return GetThumbnailFromImage(image);
			}
			return (Bitmap)null;
		}
		/// <summary>
		/// Generate a thumbnail image by scaling an input image to the proper size of the control.
		/// </summary>
		/// <param name="image"></param>
		/// <returns></returns>
		private Bitmap GetThumbnailFromImage(Image image) 
		{
			if(image == (Bitmap)null)
				return (Bitmap)null;

			int TileWidth  = (this.Width / _Count) - (2 * _Margin),	// how big are our tiles?
				TileHeight = this.Height - (2 * _Margin);

			double scale = Math.Min(						// find the correct scale factor
				(double)TileWidth / (double)image.Width,
				(double)TileHeight / (double)image.Height);

			// calculate actual thumbnail size.
			int ThumbWidth  = (int)((double)image.Width * scale),
				ThumbHeight = (int)((double)image.Height * scale);

			// ok, make the thumbnail image
			Bitmap Thumbnail = new Bitmap(ThumbWidth, ThumbHeight);
			using(Graphics G = Graphics.FromImage(Thumbnail)) 
			{
				G.DrawImage(image,new Rectangle(0,0,ThumbWidth, ThumbHeight),0,0,image.Width, image.Height, GraphicsUnit.Pixel);
			}
			return Thumbnail;
		}
		protected override void OnPaint(PaintEventArgs pe)
		{
			if(_Count == 0) 
			{
				using(SolidBrush B = new SolidBrush(this.BackColor)) 
				{
					pe.Graphics.FillRectangle(B,0,0,this.Width, this.Height);
				}
				return;
			}
			int TileWidth = this.Width / this._Count;
			for(int i = 0; i < _Count; i++) 
			{
				int X = i*TileWidth,
					Y = 0;
				if(this._Highlight && (_Selected == i))
					pe.Graphics.DrawImageUnscaled(HighlightTile, X, Y, HighlightTile.Width, HighlightTile.Height);
				else
					pe.Graphics.DrawImageUnscaled(BackgroundTile, X, Y, BackgroundTile.Width, BackgroundTile.Height);
				if(_Items[i] != (Bitmap)null) 
				{
					int XOffset = Math.Max(0, (TileWidth - 2*Margin - _Items[i].Width)/2),
						YOffset = Math.Max(0, (this.Height - 2*Margin - _Items[i].Height)/2);
					pe.Graphics.DrawImageUnscaled(_Items[i], X+_Margin+XOffset, Y+_Margin+YOffset, _Items[i].Width, _Items[i].Height);
				}
			}
			if(_Border == true) 
			{
				using(Pen P = new Pen(this.ForeColor))
				{
					pe.Graphics.DrawRectangle(P, 0, 0, this.Width-1, this.Height-1);
				}
			}
		}

		// not sure how much of this I need for a non-animated control...
		private void ImagePreviewPanel_Resize(object sender, System.EventArgs e)
		{
			CreateTiles();	// re-create background and highlight tiles for the new size.
			this.Invalidate();	// send WM_PAINT
			this.Update();		// Push paint message to head of queue.
		}

		private void ImagePreviewPanel_BackColorChanged(object sender, System.EventArgs e)
		{
			CreateTiles();
		}
	}
}
