using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace HHHHControls
{
	/// <summary>
	/// Summary description for RampSlider.
	/// </summary>
	[
		ToolboxBitmapAttribute(typeof(HHHHControls.RampSlider), "RampSlider.bmp")
	]
	public class RampSlider : System.Windows.Forms.Control
	{
		private int _Min, _Max, _Value;
		private double _MinHeightFraction;
		private Color _DarkColor;
		private SolidBrush FGBrush, BGBrush, DarkBrush;
		private Pen FGPen, BGPen, DarkPen;

		private bool _MouseIsDown;
		private int _PreviousValue;

		[
			Browsable(true),
			CategoryAttribute("RampSlider"),
			Description("Minimum value of the slider.")
		]
		public int Min 
		{
			get { return _Min; }
			set { _Min = value; }
		}

		[
		Browsable(true),
		CategoryAttribute("RampSlider"),
		Description("Maximum value of the slider.")
		]
		public int Max 
		{
			get { return _Max; }
			set { _Max = value; }
		}

		[
		Browsable(true),
		CategoryAttribute("RampSlider"),
		Description("Current value of the slider.")
		]
		public int Value 
		{
			get { return _Value; }
			set { _Value = value; UpdateDisplay(); }
		}

		[
		Browsable(true),
		CategoryAttribute("RampSlider"),
		Description("Height of the slider display at the minimum position, as a fraction of the control's height.")
		]
		public double MinHeightFraction 
		{
			get { return _MinHeightFraction; }
			set { _MinHeightFraction = value; }
		}

		[
		Browsable(true),
		CategoryAttribute("RampSlider"),
		Description("Fires when the control's value changes.")
		]
		public event EventHandler ValueChanged;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public RampSlider()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.Name = "RampSlider";
			this.Size = new Size(200, 50);
			this.SetStyle(ControlStyles.DoubleBuffer, true);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			this.SetStyle(ControlStyles.UserPaint, true);

			_Min = 0;
			_Max = 100;
			_Value = 0;
			_MinHeightFraction = 0.25;
			int R = (int)(this.ForeColor.R + this.BackColor.R) / 2,
				G = (int)(this.ForeColor.G + this.BackColor.G) / 2,
				B = (int)(this.ForeColor.B + this.BackColor.B) / 2;
			_DarkColor = Color.FromArgb(R, G, B);
			FGBrush = new SolidBrush(this.ForeColor);
			BGBrush = new SolidBrush(this.BackColor);
			DarkBrush = new SolidBrush(this._DarkColor);
			FGPen = new Pen(FGBrush, -1.0f);
			BGPen = new Pen(BGBrush, -1.0f);
			DarkPen = new Pen(DarkBrush, -1.0f);

			_MouseIsDown = false;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
				// dispose our GDI objects:
				FGBrush.Dispose();
				BGBrush.Dispose();
				DarkBrush.Dispose();
				FGPen.Dispose();
				BGPen.Dispose();
				DarkPen.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// RampSlider
			// 
			this.ForeColorChanged += new System.EventHandler(this.RampSlider_ForeColorChanged);
			this.Resize += new System.EventHandler(this.RampSlider_Resize);
			this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.RampSlider_MouseUp);
			this.BackColorChanged += new System.EventHandler(this.RampSlider_BackColorChanged);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.RampSlider_MouseMove);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.RampSlider_MouseDown);

		}
		#endregion

		protected override void OnPaint(PaintEventArgs pe)
		{
			int MinHeight = (int)((double)this.Height * (1.0 - _MinHeightFraction)),
				ValueXPos = (int)((this.Width-1) * ((double)_Value / (double)(_Max - _Min))),
				ValueYPos = MinHeight - (int)((double)MinHeight * ((double)_Value / (double)(_Max - _Min)));

			// clear the frame:
			pe.Graphics.FillRectangle(BGBrush, 0, 0, this.Width, this.Height);

			// draw rectangles.
			int NumRegions = (_Max - _Min) + 1,		// a dynamic range of 3 to 7 is (7-3)+1 = 5 possible values.
				RegionWidth = Math.Max(1, (int)((double)this.Width / (double)NumRegions));
			for(int i = 0; i < NumRegions; i++) 
			{
				int xi = (int)(((double)i * (double)this.Width) / (double)(NumRegions)),				// x value at position i
					yn = MinHeight - (int)((double)MinHeight * ((double)(i+1) / (double)(NumRegions))), // y value at position n = i+1
					RegionHeight = this.Height - yn;													// height of the rectangle

				if(i + _Min <= _Value) // this is the "on" portion
				{
					pe.Graphics.FillRectangle(FGBrush, xi, yn, RegionWidth, RegionHeight);
				} 
				else	// this is the "off" portion
				{
					pe.Graphics.FillRectangle(DarkBrush, xi, yn, RegionWidth, RegionHeight);
				}
			}
			if(RegionWidth >= 3)	// suppress separator lines if regions are too narrow.
			{
				for(int i = 1; i < NumRegions; i++)  // draw separator lines
				{
					int xi = (int)(((double)i * (double)this.Width) / (double)(NumRegions));
					pe.Graphics.DrawLine(BGPen, xi, 0, xi, this.Height);
				}
			}

			// Calling the base class OnPaint
			// base.OnPaint(pe);
		}

		public void UpdateDisplay()
		{
			this.Invalidate();	// send WM_PAINT
			this.Update();		// push WM_PAINT message to the head of the queue
		}

		private void RampSlider_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			_MouseIsDown = true;
			_PreviousValue = _Value; // remember this in case the user cancels the operation.
			RampSlider_MouseMove(sender, e); // update the value now, as we're not guaranteed to get a MouseMove event.
		}

		private void RampSlider_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(_MouseIsDown == true) // do nothing unless the mouse is down.
			{
				// update the control's value,
				_Value = (int)((double)e.X * (double)(this.Max+1) / (double)this.Width);
				_Value = Math.Min(_Value, _Max);  // bounds-check it.
				_Value = Math.Max(_Value, _Min);

				// update the display,
				UpdateDisplay();
				// notify interested parties.
				if(ValueChanged != null)
					ValueChanged(this, new EventArgs());
			}
		}

		private void RampSlider_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(_MouseIsDown == true) 
			{
				_MouseIsDown = false;
				if(e.X < 0 || e.X > this.Width || e.Y < 0 || e.Y > this.Height) 
				{
					CancelChange();
					UpdateDisplay();
				}
			}
		}

		private void CancelChange()
		{
			_Value = _PreviousValue;
			if(ValueChanged != null)
				ValueChanged(this, new EventArgs());
		}

		private void RampSlider_Resize(object sender, System.EventArgs e)
		{
			UpdateDisplay();
		}

		private void RampSlider_ForeColorChanged(object sender, System.EventArgs e)
		{
			UpdateBrushes();
		}

		private void RampSlider_BackColorChanged(object sender, System.EventArgs e)
		{
			UpdateBrushes();
		}

		// re-create our brushes and pens when colors change, and update the display.
		private void UpdateBrushes() 
		{
			if(FGBrush   != (SolidBrush)null) FGBrush.Dispose();		// get rid of the old ones.
			if(BGBrush   != (SolidBrush)null) BGBrush.Dispose();
			if(DarkBrush != (SolidBrush)null) DarkBrush.Dispose();
			if(FGPen     != (Pen)null)        FGPen.Dispose();
			if(BGPen     != (Pen)null)        BGPen.Dispose();
			if(DarkPen   != (Pen)null)        DarkPen.Dispose();

			int R = (int)(this.ForeColor.R + this.BackColor.R) / 2,
				G = (int)(this.ForeColor.G + this.BackColor.G) / 2,
				B = (int)(this.ForeColor.B + this.BackColor.B) / 2;
			_DarkColor = Color.FromArgb(R, G, B);
			FGBrush = new SolidBrush(this.ForeColor);
			BGBrush = new SolidBrush(this.BackColor);
			DarkBrush = new SolidBrush(this._DarkColor);
			FGPen = new Pen(FGBrush, -1.0f);
			BGPen = new Pen(BGBrush, -1.0f);
			DarkPen = new Pen(DarkBrush, -1.0f);

			UpdateDisplay();
		}
	}
}
